"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const card_1 = __importDefault(require("../card"));
const card_constant_1 = require("../card/card.constant");
const card_interface_1 = require("../card/card.interface");
/**
 * Represents a deck of playing cards.
 */
class Deck {
    /**
     * Represents a deck of cards.
     * @constructor
     * @param options - The options for initializing the deck.
     * @param options.deck - The deck of cards to use.
     * @param options.withJokers - Whether to include jokers in the deck. Default is false.
     */
    constructor(options) {
        this.deck = [];
        this.drawCards = [];
        this.initDeck(options);
    }
    /**
     * Retrieves the deck of cards.
     * @returns {Card[]} The deck of cards.
     */
    getDeck() {
        return this.deck;
    }
    /**
     * Initializes the deck with the specified options or creates a new deck if no options are provided.
     * @param options - The options to initialize the deck with.
     * @param options.deck - The deck of cards to use.
     * @param options.withJokers - Whether to include jokers in the deck. Default is false.
     * @returns The initialized deck.
     */
    initDeck(options) {
        if (options && options.deck) {
            this.deck = options.deck;
            return this;
        }
        const deck = [];
        for (let i = 0; i < card_constant_1.Suits.length; i++) {
            for (let x = 0; x < card_constant_1.Values.length; x++) {
                const card = new card_1.default(card_constant_1.Suits[i], card_constant_1.Values[x]);
                deck.push(card);
            }
        }
        if (options && options.withJokers) {
            deck.push(new card_1.default(card_interface_1.CARD_SUIT.RED, card_interface_1.CARD_VALUE.JOKER));
            deck.push(new card_1.default(card_interface_1.CARD_SUIT.BLACK, card_interface_1.CARD_VALUE.JOKER));
        }
        this.deck = deck;
        return this;
    }
    /**
     * Resets the deck by reinitializing it with a standard set of playing cards.
     * @returns The current instance of the Deck class.
     */
    resetDeck() {
        return this.initDeck();
    }
    /**
     * Shuffles the deck by swapping random cards multiple times.
     * @param nbShuffleTime The number of times to shuffle the deck. Default is 1000.
     * @returns The updated Deck instance with the shuffled deck.
     */
    shuffleDeck(nbShuffleTime = 1000) {
        const deck = this.deck.slice();
        for (let i = 0; i < nbShuffleTime; i++) {
            const pos1 = Math.floor((Math.random() * deck.length));
            const pos2 = Math.floor((Math.random() * deck.length));
            const tmp = deck[pos1];
            deck[pos1] = deck[pos2];
            deck[pos2] = tmp;
        }
        this.deck = deck;
        return this;
    }
    /**
     * Draws a card from the deck and adds it to the drawCards array.
     * @returns {Card | undefined} The card that was drawn from the deck, or undefined if the deck is empty.
     */
    drawCard() {
        const card = this.deck.shift();
        if (card) {
            this.drawCards.push(card);
        }
        return card;
    }
}
exports.default = Deck;
